package PDL::Logical;
$PDL::Logical::VERSION = '0.003';
use strict;
use warnings;
# Enumerable: true, false

# TODO


1;

__END__

=pod

=encoding UTF-8

=head1 NAME

PDL::Logical

=head1 VERSION

version 0.003

=head1 AUTHOR

Zakariyya Mughal <zmughal@cpan.org>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2014 by Zakariyya Mughal.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
